@@ -36,14 +36,6 @@ class WebhookTestCase(TestCase):
     def setUp(self):
         self.user = mommy.make(User, username='test_user')
         self.service = WebhookService()
-        self.webhook = mommy.make(
-            Webhook,
-            user=self.user,
-            target_url='http://example2.com/order_create',
-            secret_key='secret',
-            event_type='order-created',
-            retry_countdown_config={'choice': 'exponential', 'kwargs': {'base': 2}},
-        )
 
     def test_register_webhook(self):
         webhook = mommy.prepare(
@@ -61,29 +53,87 @@ class WebhookTestCase(TestCase):
         webhook = self.service.register_webhook(
             user=self.user, **serializer.validated_data
         )
+        self.assertEqual(webhook.event_type, 'order-created')
 
         with self.assertRaises(WebhookAlreadyRegistered):
-            webhook = self.service.register_webhook(
-                user=self.user, **serializer.validated_data
-            )
+            self.service.register_webhook(user=self.user, **serializer.validated_data)
 
     def test_update_webhook(self):
+        webhook = mommy.prepare(
+            Webhook,
+            user=self.user,
+            target_url='http://example2.com/order_create',
+            secret_key='secret',
+            event_type='order-created',
+            retry_countdown_config={'choice': 'exponential', 'kwargs': {'base': 2}},
+        )
+
+        serializer = WebhookSerializer(webhook)
+        serializer = WebhookSerializer(data=serializer.data)
+        self.assertTrue(serializer.is_valid(raise_exception=True))
+        webhook_created = self.service.register_webhook(
+            user=self.user, **serializer.validated_data
+        )
+
         update_data = {
-            'target_url': 'http://example.com/order_update',
+            'target_url': 'http://example3.com/order_update',
             'event_type': 'order-update',
             'retry_countdown_config': {
                 'choice': 'linear',
                 'kwargs': {'base': 1 * 60, 'limit': 5 * 60},
             },
         }
-        serializer = WebhookSerializer(self.webhook, data=update_data)
-        self.assertTrue(serializer.is_valid(raise_exception=True))
-        webhook = self.service.update_webhook(
-            self.webhook, self.user, **serializer.validated_data
+        webhook_updated = self.service.update_webhook(
+            webhook_created, self.user, **update_data
+        )
+
+        self.assertNotEqual(webhook.target_url, webhook_updated.target_url)
+        self.assertNotEqual(webhook.event_type, webhook_updated.event_type)
+
+        self.assertEqual(webhook_updated.target_url, update_data['target_url'])
+        self.assertEqual(webhook_updated.event_type, 'order-update')
+
+    def test_update_webhook_with_already_registered_one(self):
+        webhook = mommy.make(
+            Webhook,
+            user=self.user,
+            target_url='http://example2.com/order_create',
+            secret_key='secret',
+            event_type='order-created',
+            retry_countdown_config={'choice': 'exponential', 'kwargs': {'base': 2}},
+        )
+
+        mommy.make(
+            Webhook,
+            user=self.user,
+            target_url='http://example2.com/order_create',
+            secret_key='secret',
+            event_type='order-update',
+            retry_countdown_config={'choice': 'exponential', 'kwargs': {'base': 2}},
+        )
+
+        update_data = {'target_url': webhook.target_url, 'event_type': 'order-update'}
+
+        with self.assertRaises(WebhookAlreadyRegistered):
+            self.service.update_webhook(webhook, self.user, **update_data)
+
+        del update_data['target_url']
+
+        with self.assertRaises(WebhookAlreadyRegistered):
+            self.service.update_webhook(webhook, self.user, **update_data)
+
+        update_data = {
+            'retry_countdown_config': {
+                'choice': 'linear',
+                'kwargs': {'base': 1 * 60, 'limit': 10 * 60},
+            },
+        }
+
+        webhook_updated = self.service.update_webhook(webhook, self.user, **update_data)
+        self.assertEqual(
+            webhook_updated.retry_countdown_config['kwargs'],
+            update_data['retry_countdown_config']['kwargs'],
         )
-        self.webhook.refresh_from_db()
-        self.assertEqual(self.webhook.target_url, 'http://example.com/order_update')
-        self.assertEqual(self.webhook.event_type, 'order-update')
 
     def test_delete_webhook(self):
         webhook = mommy.make(Webhook, is_active=True)
@@ -192,6 +242,16 @@ class WhispererEventTestCase(TestCase):
         self.assertEqual(webhook_events[0].response_http_status, 500)
         self.assertIn("HTTPError", webhook_events[0].response_content)
 
+    @mock.patch('requests.post')
+    def test_unboundlocal_error(self, post_mock):
+        post_mock.side_effect = UnboundLocalError()
+        self.order.save()
+        webhook_events = WebhookEvent.objects.all()
+        self.assertEqual(len(webhook_events), 1)
+        self.assertFalse(webhook_events[0].delivered)
+        self.assertEqual(webhook_events[0].response_http_status, 500)
+        self.assertIn("", webhook_events[0].response_content)
+
     def test_undelivered_event_scanner(self):
         date1 = timezone.now() - timedelta(days=1)
         date2 = timezone.now() - timedelta(days=4)
